/*
 * Copyright (c) 1997-1998 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

#include "getproc.h"

static pid_t nextpid = 1;
extern struct vnode *rootvnode;

struct ourproc {
    struct proc _proc;	/* must be first */
    struct pcred _pcred;
    struct filedesc0 _fd;
    struct plimit _plimit;
    struct pstats _pstats;
};

oskit_error_t getproc(struct proc **procp)
{
    struct ourproc *oproc;
    oskit_principal_t *prin;
    oskit_identity_t id;
    oskit_error_t ferror;
    struct proc *proc;
    struct pcred *pcred;
    struct ucred *cred;
    struct filedesc0 *fd;
    struct plimit *limit;
    int index;

    oproc = malloc(sizeof *oproc, M_TEMP, M_WAITOK);
    if (!oproc)
	return OSKIT_ENOMEM;
    bzero(oproc, sizeof *oproc);
    proc = &oproc->_proc;

    pcred = &oproc->_pcred;
    pcred->p_refcnt = 1;
    proc->p_cred = pcred;

    ferror = oskit_get_call_context(&oskit_principal_iid, (void *)&prin);
    if (!ferror)
    {
	ferror = oskit_principal_getid(prin, &id);
	if (ferror)
	    goto bad2;
    }
    else
    {
        bzero(&id, sizeof id);    
	prin = 0;
    }

    if (id.ngroups > NGROUPS)
    {
	ferror = OSKIT_EINVAL;
	goto bad3;
    }

    cred = crget();
    if (!cred)
    {
	ferror = OSKIT_ENOMEM;
	goto bad3;
    }

    cred->cr_uid = id.uid;
    cred->cr_gid = id.gid;
    cred->cr_ngroups = id.ngroups;
    for (index = 0; index < id.ngroups; index++)
	cred->cr_groups[index] = id.groups[index];

    pcred->p_ruid = pcred->p_svuid = id.uid;
    pcred->p_rgid = pcred->p_svgid = id.gid;
    
    if (id.groups)
	FREE(id.groups, M_TEMP);
    if (prin)
	oskit_principal_release(prin);

    proc->p_ucred = cred;

    fd = &oproc->_fd;
    fd->fd_fd.fd_refcnt = 1;
    fd->fd_fd.fd_cmask = 0;	/* umask is applied by client.  */
    fd->fd_fd.fd_ofileflags = fd->fd_dfileflags;
    fd->fd_fd.fd_nfiles = NDFILE;
    proc->p_fd = &fd->fd_fd;

    fd->fd_fd.fd_rdir = rootvnode = 0;

    limit = &oproc->_plimit;
    limit->p_refcnt = 1;
    for (index = 0; index < RLIM_NLIMITS; index++)
	limit->pl_rlimit[index].rlim_cur = 
		limit->pl_rlimit[index].rlim_max = RLIM_INFINITY;
    proc->p_limit = limit;

    proc->p_stats = &oproc->_pstats;

    if (nextpid == 0)
	nextpid = 1;
    proc->p_pid = nextpid++;    

    *procp = proc;
    curproc = proc;
    return 0;

 bad3:
    if (id.groups)
	free(id.groups, M_TEMP);
 bad2:
    if (prin)
	oskit_principal_release(prin);
 bad:
    free(oproc, M_TEMP);
    return ferror;
}


void prfree(struct proc *proc)
{
    crfree(proc->p_ucred);
    free(proc, M_TEMP);
    curproc = 0;
}
